package aexp.gyrocapture;

import java.util.List;

import android.app.Activity;
import android.content.Intent;
import android.content.ServiceConnection;
import android.content.ComponentName;
import android.content.Context;
import android.hardware.Sensor;
import android.hardware.SensorManager;
import android.media.AudioManager;
import android.os.Bundle;
import android.os.DeadObjectException;
import android.os.RemoteException;
import android.os.IBinder;
import android.util.Log;
import android.view.View;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.TextView;
import android.widget.SeekBar;
import android.widget.Button;

public class GyroCapture extends Activity 
{
	static final String LOG_TAG = "GYROCAPTURE";
	static final String SAMPLING_SERVICE_ACTIVATED_KEY = "samplingServiceActivated";
	static final String STEPCOUNT_KEY = "sampleCounter";

    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);
		Log.d( LOG_TAG, "onCreate" );
		if( savedInstanceState != null ) {
			sampleCounterText = savedInstanceState.getString( STEPCOUNT_KEY );
			samplingServiceActivated = savedInstanceState.getBoolean( SAMPLING_SERVICE_ACTIVATED_KEY, false );
        } else
            samplingServiceActivated = false;
        Log.d( LOG_TAG, "onCreate; samplingServiceActivated: "+samplingServiceActivated );
		bindSamplingService();
        setContentView( R.layout.main );
		SensorManager sensorManager = 
				(SensorManager)getSystemService( SENSOR_SERVICE  );
		updateSensorName( R.id.accelsensorname,
				sensorManager,
				Sensor.TYPE_ACCELEROMETER,
				"Accelerometer");
		updateSensorName( R.id.gyrosensorname,
				sensorManager,
				Sensor.TYPE_GYROSCOPE,
				"Gyroscope");
		sampleCounterTV = (TextView)findViewById( R.id.samplecounter );
		if( sampleCounterText != null ) {
			sampleCounterTV.setText( sampleCounterText );
			sampleCounterTV.setVisibility( View.VISIBLE );
        }
		CheckBox cb = (CheckBox)findViewById( R.id.cb_sampling );
        if( samplingServiceActivated )
            cb.setChecked( true );
        else
            stopSamplingService();
		cb.setOnCheckedChangeListener( new CompoundButton.OnCheckedChangeListener() {
			public void onCheckedChanged(CompoundButton  buttonView, boolean isChecked) {
				if( isChecked ) {
					Log.d( LOG_TAG, "sampling activated" );
					samplingServiceActivated = true;
					startSamplingService();
			        sampleCounterTV.setText( sampleCounterText );
					sampleCounterTV.setVisibility( View.VISIBLE );
				} else {
					Log.d( LOG_TAG, "sampling deactivated" );
					samplingServiceActivated = false;
					sampleCounterTV.setVisibility( View.INVISIBLE );
					stopSamplingService();
				}
			}
		});


    }

	protected void onSaveInstanceState(Bundle outState) {
		super.onSaveInstanceState( outState );
		Log.d( LOG_TAG, "onSaveInstanceState" );
		outState.putBoolean( SAMPLING_SERVICE_ACTIVATED_KEY, samplingServiceActivated ); 
		if( sampleCounterText != null )
			outState.putString( STEPCOUNT_KEY, sampleCounterText );
	}

	protected void onDestroy() {
		super.onDestroy();
		Log.d( LOG_TAG, "onDestroy" );
		releaseSamplingService();
	}

	private void updateSensorName(
			int tvID,
			SensorManager sensorManager,
			int sensorType,
			String sensorName) {
		TextView tv = (TextView)findViewById( tvID );
		List<Sensor> sensors = 
				sensorManager.
					getSensorList( 
						sensorType);
		StringBuffer sb = new StringBuffer();
		if( sensors.size() == 0 ) {
			sb.append( sensorName );
			sb.append( ": N/A");
		} else {
			Sensor s = sensors.get(0);
			sb.append( s.getName());
		}
		tv.setText( new String( sb ));
	}
	
	private void startSamplingService() {
		if( samplingServiceRunning )	// shouldn't happen
			stopSamplingService();
        sampleCounterText = "0";
        Intent i = new Intent();
        i.setClassName( "aexp.gyrocapture","aexp.gyrocapture.SamplingService" );
        startService( i );
		samplingServiceRunning = true;				
	}

	private void stopSamplingService() {
		Log.d( LOG_TAG, "stopSamplingService" );
		if( samplingServiceRunning ) {
			stopSampling();
			samplingServiceRunning = false;
		}
	}

	private void bindSamplingService() {
		samplingServiceConnection = new SamplingServiceConnection();
		Intent i = new Intent();
		i.setClassName( "aexp.gyrocapture", "aexp.gyrocapture.SamplingService" );
		bindService( i, samplingServiceConnection, Context.BIND_AUTO_CREATE);
	}

	private void releaseSamplingService() {
		releaseCallbackOnService();
		unbindService( samplingServiceConnection );	  
		samplingServiceConnection = null;
	}


    private void setCallbackOnService() {
		if( samplingService == null )
			Log.e( LOG_TAG, "setCallbackOnService: Service not available" );
		else {
			try {
				samplingService.setCallback( iSteps.asBinder() );
			} catch( DeadObjectException ex ) {
				Log.e( LOG_TAG, "DeadObjectException",ex );
			} catch( RemoteException ex ) {
				Log.e( LOG_TAG, "RemoteException",ex );
			}
		}
	}

    private void releaseCallbackOnService() {
		if( samplingService == null )
			Log.e( LOG_TAG, "releaseCallbackOnService: Service not available" );
		else {
			try {
				samplingService.removeCallback();
			} catch( DeadObjectException ex ) {
				Log.e( LOG_TAG, "DeadObjectException",ex );
			} catch( RemoteException ex ) {
				Log.e( LOG_TAG, "RemoteException",ex );
			}
		}
	}

    private void updateSamplingServiceRunning() {
		if( samplingService == null )
			Log.e( LOG_TAG, "updateSamplingServiceRunning: Service not available" );
		else {
			try {
				samplingServiceRunning = samplingService.isSampling();
			} catch( DeadObjectException ex ) {
				Log.e( LOG_TAG, "DeadObjectException",ex );
			} catch( RemoteException ex ) {
				Log.e( LOG_TAG, "RemoteException",ex );
			}
		}
	}

    private void stopSampling() {
		Log.d( LOG_TAG, "stopSampling" );
		if( samplingService == null )
			Log.e( LOG_TAG, "stopSampling: Service not available" );
		else {
			try {
				samplingService.stopSampling();
			} catch( DeadObjectException ex ) {
				Log.e( LOG_TAG, "DeadObjectException",ex );
			} catch( RemoteException ex ) {
				Log.e( LOG_TAG, "RemoteException",ex );
			}
		}
	}

    private IGyroCapture.Stub iSteps 
				= new IGyroCapture.Stub() {

		@Override
		public void sampleCounter(int count) throws RemoteException {
			Log.d( LOG_TAG, "sample count: "+count );
			sampleCounterText = Integer.toString( count );
			sampleCounterTV.setText( sampleCounterText );			
		}
    };

    private ISamplingService samplingService = null;
    private SamplingServiceConnection samplingServiceConnection = null;
	private boolean samplingServiceRunning = false;
    private boolean samplingServiceActivated = false;
	private TextView sampleCounterTV;
	private String sampleCounterText = null;

    class SamplingServiceConnection implements ServiceConnection {
        public void onServiceConnected(ComponentName className, 
			IBinder boundService ) {
			Log.d( LOG_TAG, "onServiceConnected" );        	
        	samplingService = ISamplingService.Stub.asInterface((IBinder)boundService);
	    	setCallbackOnService();
			updateSamplingServiceRunning();
/*
            if( !samplingServiceRunning )
                startSamplingService();
*/                
		    CheckBox cb = (CheckBox)findViewById( R.id.cb_sampling );
            cb.setChecked( samplingServiceRunning );
		 	Log.d( LOG_TAG,"onServiceConnected" );
        }

        public void onServiceDisconnected(ComponentName className) {
        	samplingService = null;
			Log.d( LOG_TAG,"onServiceDisconnected" );
        }
    };


}
