package aexp.gyrocapture;

import android.app.Service;
import android.content.Intent;
import android.content.Context;
import android.content.SharedPreferences;
import android.content.IntentFilter;
import android.content.BroadcastReceiver;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.os.Environment;
import android.os.PowerManager;
import android.os.IBinder;
import android.os.RemoteException;
import android.os.DeadObjectException;
import android.hardware.SensorManager;
import android.util.Log;
import android.util.LogPrinter;
import java.io.PrintWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.List;
import java.util.Date;

public class SamplingService extends Service implements SensorEventListener {
	static final String LOG_TAG = "GYROCAPTURE";
	static final boolean DEBUG_GENERAL = true;
	static final int SAMPLECTR_MOD = 1000;
	
	public int onStartCommand(Intent intent, int flags, int startId) {
		super.onStartCommand( intent, flags, startId );
		Log.d( LOG_TAG, "onStartCommand" );
		stopSampling();		// just in case the activity-level service management fails
        rate = SensorManager.SENSOR_DELAY_FASTEST;
		sensorManager = (SensorManager)getSystemService( SENSOR_SERVICE  );
		startSampling();
		Log.d( LOG_TAG, "onStartCommand ends" );
		return START_NOT_STICKY;
	}

	public void onDestroy() {
		super.onDestroy();
		Log.d( LOG_TAG, "onDestroy" );
		stopSampling();
	}

	public IBinder onBind(Intent intent) {
		return serviceBinder;	// cannot bind
	}

// SensorEventListener
    public void onAccuracyChanged (Sensor sensor, int accuracy) {
    }

    public void onSensorChanged(SensorEvent sensorEvent) {
		processSample( sensorEvent );
    }


	private void stopSampling() {
		if( !samplingStarted )
			return;
        if( sensorManager != null ) {
			Log.d( LOG_TAG, "unregisterListener/SamplingService" );
            sensorManager.unregisterListener( this );
		}
        if( captureFile != null ) {
            captureFile.close();
			captureFile = null;
        }
		samplingStarted = false;
	}

	private void startSampling() {
		if( samplingStarted )
			return;
		sampleCounter = 0;
       	List<Sensor> sensors = sensorManager.getSensorList( Sensor.TYPE_ACCELEROMETER  );
      	accelSensor = sensors.size() == 0 ? null : sensors.get( 0 );
       	sensors = sensorManager.getSensorList( Sensor.TYPE_GYROSCOPE );
      	gyroSensor = sensors.size() == 0 ? null : sensors.get( 0 );
       	
      	if( ( accelSensor != null ) && ( gyroSensor != null ) ) {
			Log.d( LOG_TAG, "registerListener/SamplingService" );
           	sensorManager.registerListener( 
                            this, 
                            accelSensor,
                            rate );
           	sensorManager.registerListener( 
                    this, 
                    gyroSensor,
                    rate );
		} else {
			Log.d( LOG_TAG, 
				"Sensor(s) missing: accelSensor: "+
				accelSensor+
				"; gyroSensor: "+
				gyroSensor);
		}
      	captureFile = null;
      	File captureFileName = new File( 
      			Environment.getExternalStorageDirectory(), 
      			"capture.csv" );
      	try {
      		captureFile = new PrintWriter( new FileWriter( captureFileName, false ) );
      	} catch( IOException ex ) {
      		Log.e( LOG_TAG, ex.getMessage(), ex );
      	}
      	samplingStarted = true;
	}


	private void updateSampleCounter() {
		++sampleCounter;
		if( ( sampleCounter % SAMPLECTR_MOD ) == 0 ) {
			if( iGyroCapture == null )
				Log.d( 
					LOG_TAG, 
					"updateSampleCounter() callback: cannot call back (sampleCounter: "+
					sampleCounter+
					")" );
			else {
				Log.d( 
					LOG_TAG, 
					"updateSampleCounter() callback: sampleCounter: "+
					sampleCounter );
				try {
					iGyroCapture.sampleCounter( sampleCounter );
				} catch( DeadObjectException ex ) {
					Log.e( LOG_TAG,"step() callback", ex );
				} catch( RemoteException ex ) {
					Log.e( LOG_TAG, "RemoteException",ex );
				}
			}
		}
	}

	private void processSample( SensorEvent sensorEvent ) {
		float values[] = sensorEvent.values;
		if( values.length < 3 )
				return;
		if( captureFile != null ) {
			String sensorName = "n/a";
			if( sensorEvent.sensor == accelSensor )
				sensorName="accel";
			else
			if( sensorEvent.sensor == gyroSensor )
				sensorName = "gyro";
			captureFile.println( sensorEvent.timestamp+
					","+
					sensorName+
					","+
					values[0]+
					","+
					values[1]+
					","+
					values[2]);
		}
		updateSampleCounter();
	}
	
    private IGyroCapture iGyroCapture = null;
    private final ISamplingService.Stub serviceBinder = 
			new ISamplingService.Stub() {
		public void setCallback( IBinder binder ) {
			Log.d( LOG_TAG,"setCallback" );
			iGyroCapture = IGyroCapture.Stub.asInterface( binder );
      	}

		public void removeCallback() {
			Log.d( LOG_TAG, "removeCallback" );
			iGyroCapture = null;
		}

		public boolean isSampling() {
			return samplingStarted;
		}

		public void stopSampling() {
			Log.d( LOG_TAG, "stopSampling" );
			SamplingService.this.stopSampling();
			stopSelf();
		}

    };


    private int sampleCounter;
    private int rate;
    private SensorManager sensorManager;
    private PrintWriter captureFile;
	private Sensor accelSensor;
	private Sensor gyroSensor;
	private boolean samplingStarted = false;

}

